/* TestMultiServer.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2006
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestMultiServer.java#5 $
 * 
 * Running this example requires several servers connected to the 
 * same database.
 * Example of configuration, using the file 
 * c:\tpmfos files\global\rad\config.csv:
 * 
 * HostName;Interfaces;DbName;DbUser;DbPass;MasterIP;MasterDbName;MasterDbUser;MasterDbPass;BinDir;Description;Reporting;AutoSync;PollInterval;DebugLevel
 * pc000c29ad7134;192.168.168.17;"[DRIVER={SQL Server};SERVER=rembomaster;UID=radusr;PWD=hidden;DATABASE=RadProxy]";;;SELF;;;;c:/temp/;embedded;;;1;4
 * pc000c29b80e4e;192.168.168.133;"[DRIVER={SQL Server};SERVER=rembomaster;UID=radusr;PWD=hidden;DATABASE=RadProxy]";;;192.168.168.17;;;;c:/temp/;s1;;;1;4
 * pc000c29296cc6;192.168.168.128;"[DRIVER={SQL Server};SERVER=rembomaster;UID=radusr;PWD=hidden;DATABASE=RadProxy]";;;192.168.168.17;;;;c:/temp/;s2;;;1;4
 *
 * In this example there is one master server on 192.168.168.17 and two 
 * slave servers. The following scenario is implemented:
 * 1) creation of an object (activity template) on the first slave
 * 2) move the object to the master (change of scope)
 * 3) synchronize the second slave
 */
package examples;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.logging.Level;

import com.ibm.rbapi.RBBootServer;
import com.ibm.rbapi.RBBootServerRecs;
import com.ibm.rbapi.RBOSRestoreTemplate;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.exception.RBTimeoutException;

public class TestMultiServer {

    public static void main(String[] args) throws UnknownHostException,
            IOException, InterruptedException, RBServerException,
            RBObjectNotFoundException, RBTimeoutException 
    {

        if (args.length < 3) {
            System.out.println("usage: TestMultiServer <ip> <port> <password>\n");
            return;
        }

        System.out.println("TestMultiServer starting. We will test getting the "
                +"list of boot servers.");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        // RBServer rbServer = new RBServer(ip, port, secret, Level.ALL);
        RBServer rbServer = new RBServer(ip, port, secret);
        // Abort if the server is not ready after 15 seconds
        rbServer.isReady(15000);

        // Display list of known boot servers
        RBBootServerRecs recs = rbServer.getBootServers("", "");
        int limit = recs.size();
        RBBootServer[] servers = recs.subList(0, limit);
        RBBootServer masterDescr = null; // remember master
        for (int i=0; i<limit; i++) {
            RBBootServer current = servers[i];
            System.out.println("bootserv["+i+"]:\t"
                    +current.getInterface0()+"\t"
                    +current.getDescription()+"\t"
                    +"master:"+current.getMasterIP()+"\t"
                    +"last:"+current.getLastActivity()+"\t"
                    +"\n\t\tsyncevent:"+current.getSyncEvent()+"\t"
                    +"syncmsg:\""+current.getSyncMessage()+"\"\t"
                    );
            if (current.getMasterIP().equals("SELF")) {
                masterDescr = current;
            }
        }
        
        // Test the replication mechanism
        // This part requires a master and a slave server
        if (masterDescr == null) {
            System.out.println("No master server found to run the whole "
                    +"test (MasterIP=SELF)");
            return;
        }
 
        RBBootServer firstSlave = null; // look for slaves
        RBBootServer otherSlave = null;
        int i;
        for (i=0; i<limit; i++) {
            RBBootServer current = servers[i];
            if (current.getMasterIP().equals(masterDescr.getInterface0())) {
                firstSlave = current;
                i++;
                break;
            }
        }
        for (; i<limit; i++) {
            RBBootServer current = servers[i];
            if (current.getMasterIP().equals(masterDescr.getInterface0())) {
                otherSlave = current;
                break;
            }
        }

        if (firstSlave == null) {
            System.out.println("No slave server found to run the whole test "
                    +"(MasterIP="+masterDescr.getInterface0()+")");
            return;
        }

        System.out.println("\nMaster will be "+masterDescr.getInterface0()
                +", slave will be "+firstSlave.getInterface0());
        
        // Create a template on the slave
        RBServer slaveServer = new RBServer(firstSlave.getInterface0(),port,secret,Level.INFO);
        RBOSRestoreTemplate tpl = slaveServer.registerOSRestoreTemplate(
                "TestMultiServer",false, false, false);
        
        // Print the scope
        System.out.println("\nA new template has been created on "
                +firstSlave.getInterface0());
        System.out.println("The initial scope of the template is "
                +tpl.getScope());
        
        // Update the scope
        tpl.setNewScope(masterDescr);
        tpl.update();
        System.out.println("After update Scope="+tpl.getScope()
                +" NewScope="+tpl.getNewScope());
        
        // Ask ***** master ***** to refresh its files
        // For the moment, the process is slow and not convenient
        // This will be improved in a future version, using an activity
        masterDescr.setFileRefreshTrigger();
        masterDescr.update();
        System.out.println("\nFile refresh requested on "+masterDescr.getInterface0());
        System.out.println("It should start within 1 minute");
        
        // Wait for start
        while (masterDescr.isSetFileRefreshTrigger()) {
            System.out.println("Synchronization has not started, sleep 5 secs...");
            Thread.sleep(5000);
            masterDescr.refresh();
        }
        System.out.println("\nSynchronization in progress");
        System.out.println("Full check can last several minutes");

        // Wait for completion
        while (masterDescr.getSyncMessage().length()>0) {
            System.out.println("Message:\""+masterDescr.getSyncMessage()+"\", sleep 5 secs...");
            Thread.sleep(5000);
            masterDescr.refresh();
        }
        System.out.println("\nSynchronization completed");
        tpl.refresh();
        System.out.println("After synchronization Scope="+tpl.getScope()
                +" NewScope="+tpl.getNewScope());
               
        // Ask ***** other slave ***** to refresh its files
        if (otherSlave == null) {
            System.out.println("\nYou should add a second slave below "
                    +masterDescr.getInterface0()+" to run the full test.");
        } else {
            otherSlave.setFileRefreshTrigger();
            otherSlave.update();
            System.out.println("\nFile refresh requested on "+otherSlave.getInterface0());
            System.out.println("It should start within 1 minute");

            // Wait for start
            while (otherSlave.isSetFileRefreshTrigger()) {
                System.out.println("Synchronization has not started, sleep 5 secs...");
                Thread.sleep(5000);
                otherSlave.refresh();
            }
            System.out.println("\nSynchronization in progress");
            System.out.println("Full check can last several minutes");

            // Wait for completion
            while (otherSlave.getSyncMessage().length()>0) {
                System.out.println("Message:\""+otherSlave.getSyncMessage()+"\", sleep 5 secs...");
                Thread.sleep(5000);
                otherSlave.refresh();
            }
            System.out.println("\nSynchronization completed");
        }

        // Remove the template
        /*
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        System.out.println("\nPress enter to continue.");
        inReader.readLine();
        */
        
        slaveServer.removeTemplate(tpl, true);
        System.out.println("\nTest complete.");
    }

}
